@@ -33,17 +33,10 @@ use parent qw(WebService::ILS::OverDrive);
 
 use constant CIRCULATION_API_URL => "http://patron.api.overdrive.com/";
 use constant TEST_CIRCULATION_API_URL => "http://integration-patron.api.overdrive.com/";
-use constant NON_LOCKABLE_FORMATS => ['ebook-overdrive', 'audiobook-overdrive', 'ebook-mediado'];
 use constant OAUTH_BASE_URL => "https://oauth.overdrive.com/";
 use constant TOKEN_URL => OAUTH_BASE_URL . 'token';
 use constant AUTH_URL => OAUTH_BASE_URL . 'auth';
 
-sub format_is_lockable {
-    my $self = shift;
-    my $format = shift or croak "No format";
-    return !scalar( grep { $_ eq $format } @{ NON_LOCKABLE_FORMATS() } );
-}
-
 =head1 CONSTRUCTOR
 
 =head2 new (%params_hash or $params_hashref)
@@ -370,7 +363,11 @@ sub remove_hold {
     );
 }
 
-# Checkouts
+=head2 checkouts ()
+
+For formats see C<checkout_formats()> below
+
+=cut
 
 my %CHECKOUTS_XLATE = (
     totalItems => 'total',
@@ -385,12 +382,29 @@ sub checkouts {
     my $res = $self->_result_xlate($checkouts, \%CHECKOUTS_XLATE);
     $res->{items} = [
         map {
+            my $item = $self->_checkout_item_xlate($_);
+            my $formats = delete ($_->{formats});
+            my $actions = delete ($_->{actions});
             my $metadata = $self->_item_metadata($_);
-            if (my $formats = $metadata->{formats}) {
-                $metadata->{formats} = [ grep { $self->format_is_lockable($_) } @$formats ];
-            };
-            my %item = ( %{ $self->_checkout_item_xlate($_) }, %$metadata );
-            \%item;
+            if ($formats) {
+                $formats = $self->_formats_xlate($item->{id}, $formats);
+            }
+            else {
+                $formats = {};
+            }
+            if ($actions) {
+                if (my $format_action = $actions->{format}) {
+                    foreach (@{$format_action->{fields}}) {
+                        next unless $_->{name} eq "formatType";
+
+                        foreach my $format (@{$_->{options}}) {
+                            $formats->{$format} = undef unless exists $formats->{$format};
+                        }
+                        last;
+                    }
+                }
+            }
+            my $i = {%$item, %$metadata, formats => $formats}; # we need my $i, don't ask me why...
         } @$items
     ];
     return $res;
@@ -447,7 +461,11 @@ sub checkout {
 
 =head2 checkout_formats ($item_id)
 
-=head3 Returns an array of available checked out formats
+=head3 Returns a hashref of available title formats and immediate availability
+
+  { format => available, ... }
+
+If format is not immediately available it must be locked first
 
 =cut
 
@@ -457,13 +475,35 @@ sub checkout_formats {
 
     my $formats = $self->native_checkout_formats($id) or return;
     $formats = $formats->{'formats'} or return;
-    my @ret;
+    return $self->_formats_xlate($id, $formats);
+}
+
+sub _formats_xlate {
+    my $self = shift;
+    my $id = shift or croak "No item id";
+    my $formats = shift or croak "No formats";
+
+    my %ret;
     my $id_uc = uc $id;
     foreach (@$formats) {
         die "Non-matching item id\nExpected $id\nGot $_->{reserveId}" unless uc($_->{reserveId}) eq $id_uc;
-        push @ret, $_->{formatType};
+        my $format = $_->{formatType};
+        my $available;
+        if (my $lt = $_->{linkTemplates}) {
+            $available = grep /^downloadLink/, keys %$lt;
+        }
+        $ret{$format} = $available;
     }
-    return \@ret;
+    return \%ret;
+}
+
+sub is_lockable {
+    my $self = shift;
+    my $checkout_formats = shift or croak "No checkout formats";
+    while (my ($format, $available) = each %$checkout_formats) {
+        return 1 unless $available;
+    }
+    return 0;
 }
 
 =head2 lock_format ($item_id, $format)
@@ -676,8 +716,6 @@ sub native_lock_format {
     my $item_id = shift or croak "No item id";
     my $format = shift or croak "No format";
 
-    croak "Format $format not lockable" unless $self->format_is_lockable($format);
-
     my $url = $self->circulation_action_url("/checkouts/$item_id/formats");
     return $self->with_json_request(
         \&_basic_callback,
